import axios from 'axios'
import oidc from 'oidc-client'
import handleApiError from '../handleApiError'

const oidcmgr = new oidc.UserManager({
  userStore: new oidc.WebStorageStateStore(),
  authority: process.env.VUE_APP_API_STEERING_OIDC_URI,
  client_id: process.env.VUE_APP_OIDC_CLIENT_ID,
  redirect_uri: process.env.VUE_APP_API_STEERING_OIDC_REDIRECT_URI,
  silent_redirect_uri: process.env.VUE_APP_API_STEERING_OIDC_REDIRECT_URI_SILENT,
  accessTokenExpiringNotificationTime: process.env.VUE_APP_API_STEERING_OIDC_EXPIRE_NOTIFICATION,
  response_type: 'id_token token',
  scope: 'openid profile email username aura_shows',
  post_logout_redirect_uri: process.env.VUE_APP_API_STEERING_OIDC_REDIRECT_URI_POSTLOGOUT,
  loadUserInfo: true
})

const state = {
  user: {
    name: '',
    email: '',
    access_token: '',
    expires_at: 0,
    logged_in: false,
    steeringUser: null
  },
  users: [],
  userOIDC: null,
  tank: {
    session_token: '',
  },
}

const getters = {
  access_token: state => state.user.access_token,
  user: state => state.user,
  users: state => state.users,
}

const mutations = {
  setUserProperties (state, user) {
    state.userOIDC = user
    state.user.logged_in = true
    state.user.name = user.profile.nickname
    state.user.email = user.profile.email
    state.user.access_token = user.access_token
  },

  clearUserProperties (state) {
    state.userOIDC = null
    state.user.logged_in = false
    state.user.name = ''
    state.user.email = ''
    state.user.access_token = ''
  },

  setSteeringUser (state, user) {
    state.user.steeringUser = {...user}
  },

  setUsers (state, users) {
    state.users = users
  },

  setAccessToken (state, token) {
    state.user.access_token = token
  },

  signOut (state) {
    state.user.logged_in = false
  },

  setTankToken (state, token) {
    state.tank.session_token = token
  },

}

const actions = {
  fetchSteeringUser (ctx) {
    axios.get(process.env.VUE_APP_API_STEERING + 'users/', {
      withCredentials: true,
      headers: { 'Authorization': 'Bearer ' + ctx.state.user.access_token }
    }).then(response => {
      if (response.data.length === 0) {
        alert('No user profile data provided by steering backend!')
      } else if (response.data.length === 1) {
        ctx.commit('setSteeringUser', response.data[0])
      } else {
        // in case we are a superuser, we get all users returned
        // so we have to iterate through the user list to find out own profile
        for (let u in response.data) {
          if (response.data[u].username === ctx.state.user.name) {
            ctx.commit('setSteeringUser', response.data[u])
            break
          }
        }
      }
    }).catch(error => {
      this.$log.error(error.response.status + ' ' + error.response.statusText)
      this.$log.error(error.response)
      alert('Error: could not fetch user object from the steering backend. See console for details.')
    })
  },

  fetchUsers (ctx, data) {
    ctx.commit('setUsers', [])
    axios.get(process.env.VUE_APP_API_STEERING + 'users', {
      withCredentials: true,
      headers: { 'Authorization': 'Bearer ' + ctx.state.user.access_token }
    }).then(response => {
      ctx.commit('setUsers', response.data)
      if (data && typeof(data.callback) === 'function') { data.callback() }
    }).catch(error => {
      handleApiError(this, error, 'could not load steering users')
      if (data && typeof(data.callbackCancel) === 'function') { data.callbackCancel() }
    })
  },

  oidcInit (ctx) {
    // TODO: remove oidc logging after thorough testing
    oidc.Log.logger = console
    this.$log.debug('Initializing oidc client')

    oidcmgr.events.addAccessTokenExpiring(() => {
      this.$log.debug('Starting silent access_token renewal')
      oidcmgr.signinSilent().then(user => {
        this.$log.debug('Renewed access token:', user.access_token)
        ctx.commit('setAccessToken', user.access_token)
      }).catch(err => {
        this.$log.error(err)
        alert('Your OpenID access token could not be renewed automatically.\n' +
          'You will be logged out in ' +
          process.env.VUE_APP_API_STEERING_OIDC_EXPIRE_NOTIFICATION +
          ' seconds.\nFor details check the console.')
      })
    })

    oidcmgr.events.addAccessTokenExpired(() => {
      this.$log.debug("OIDC token has expired. Logging out.")
      ctx.commit('signOut')
    })

    oidcmgr.getUser().then(user => {
      if (user == null) {
        ctx.commit('clearUserProperties')
      } else {
        let expiry = new Date(user.expires_at * 1000)
        // if token we have a token but it has already expired, log out the user
        if (expiry <= new Date()) {
          this.$log.debug('Token has already expired:', expiry.toString())
          this.$log.debug('Logging out the user locally')
          ctx.dispatch('signoutRedirect')
        }
        // only if the token is still valid we use the existing user info
        // and establish a session with tank
        else {
          ctx.commit('setUserProperties', user)
          ctx.dispatch('fetchSteeringUser')
          // TODO: remove debug info after thorough testing
          this.$log.debug('Token will expire at:', expiry.toString())
          this.$log.debug('Access token', user.access_token)

          // now that we have a valid token, we can create a session with tank
          let uri = process.env.VUE_APP_TANK + 'auth/session'
          let payload = {
            backend: "oidc",
            arguments: {
              access_token: user.access_token,
              token_type: "Bearer"
            }
          }
          axios.post(uri, payload , {
            headers: {
              'Authorization': 'Bearer ' + user.access_token,
            }
          }).then(response => {
            this.$log.debug('tank session token:', response.data.token)
            ctx.commit('setTankToken', response.data.token)
          }).catch(error => {
            handleApiError(this, error, 'could not get a session with tank')
          })
        }
      }
    }).catch(err => {
      this.$log.debug(err)
      alert('Error: could not fetch OIDC user object. See console for details.')
    })
  },

  signinRedirect () {
    oidcmgr.signinRedirect().catch(err => {
      alert('Error: something went wrong when signing in. See console for details.')
      this.$log.error(err)
    })
  },

  signoutRedirect (ctx) {
    oidcmgr.signoutRedirect().then(resp => {
      ctx.commit('signOut')
      this.$log.debug('signed out', resp)
    }).catch(err => {
      this.$log.error(err)
      alert('Error: something went wrong when logging out. See console for details.')
    })
  },
}

export default {
  namespaced: true,
  state,
  getters,
  actions,
  mutations,
}
